import math
import random
import cocos
import pyglet
from pyglet.window import key, mouse
from vec2d import Vec2d
from cocos.actions.interval_actions import Delay
from cocos.actions import *

import Actor
from Bird import Bird

from Context import ActorBuildData, Animation, Z

class Egg(Actor.Actor):
    
    def __init__(self, context, build_data):
        super(Egg, self).__init__(context, build_data)
        
        if build_data.genes == None:
            self.genes = {}
            self.genes["jumping"] = random.randint(1,10)
            self.genes["flying"] = random.randint(1,10)
            self.genes["nomminess"] = random.randint(1,10)
            self.genes["reproducibility"] = random.randint(1,10)
            self.genes["cleverness"] = random.randint(1,10)
        else:
            self.genes = build_data.genes
        
        
        im  = pyglet.image.load( "../data/eggs01.png" )
        self.imsplit = range(6)        
                
        for i in range(6):
            self.imsplit[i] = im.get_region(i*im.width/6, 0, im.width/6, im.height)
        
        self.sprite = None
        
        self.context = context
        self.do( 
                Delay(1) \
                 + CallFunc(self.show) \
                 + Delay(1) \
                 + CallFunc(self.crack) \
                 + Delay(1) \
                 + CallFunc(self.splitAndHatch) \
                 + Delay(3) \
                 + CallFunc( self.fade ) \
                 + Delay(3) \
                 + CallFunc( self.die ) )
        
        self.t = 0.0
        self.dofade = False
        self.bird = None
        
    def destroy(self, context):
        context.rootnode.remove(self.sprite)
        self.bird.state = "Walking"
        
    def update(self, context):
        super(Egg,self).update(context)
        
        if self.sprite is not None:
            self.sprite.x, self.sprite.y = self.x, self.y
        
        if self.dofade:
            self.sprite.opacity *= 0.98
            #self.bird.birdnode.scale = self.bird.birdnode.scale + (1.0 - self.bird.birdnode.scale) * 0.001
            
        if self.bird is not None and self.bird.state == "Hatching":
            self.context.rootnode.remove(self.bird.birdnode)
            self.bird.y += context.dt * 6
            #self.bird.birdnode.
            self.context.rootnode.add(self.bird.birdnode, z = context.getActorZ(self) -  0.01)
        
    def show(self):
        self.sprite = cocos.sprite.Sprite( self.imsplit[0] ) 
        self.sprite.x, self.sprite.y = self.x, self.y
        self.context.rootnode.add(self.sprite, z = self.context.getActorZ(self))
        
    def crack(self):
        self.context.rootnode.remove(self.sprite)
        self.sprite = cocos.sprite.Sprite( self.imsplit[1] ) 
        self.sprite.x, self.sprite.y = self.x, self.y
        self.context.rootnode.add(self.sprite, z = self.context.getActorZ(self))
        
    def splitAndHatch(self):
        self.context.rootnode.remove(self.sprite)
        self.sprite = cocos.sprite.Sprite( self.imsplit[2] ) 
        self.sprite.x, self.sprite.y = self.x, self.y
        self.context.rootnode.add(self.sprite, z = self.context.getActorZ(self))
        
        build_data = ActorBuildData()
        build_data.pos = Vec2d(self.x - 10, self.y - 30)
        build_data.image = None
        build_data.genes = self.genes
        build_data.state = "Hatching"
        self.bird = self.context.level.createActor( Bird, self.context, build_data)
        self.bird.birdnode.scale = 0.6
        
    def fade(self):
        self.bird.jump()
        self.dofade = True
        
    def die(self):
        self.context.actors_to_remove.append(self)
        
    def hatch(self):
        #print "hatch!"
        build_data = ActorBuildData()
        
        build_data.pos = Vec2d(self.x, self.y)
        build_data.image = None
        
        build_data.genes = {}
        
        build_data.genes = self.genes
            
        self.context.level.createActor( Bird, self.context, build_data)
        
        # todo: add code to remove egg and hatch-animation