import cocos

from cocos.scenes.transitions import *
from cocos.actions import *

from pyglet.window import key, mouse

import CreditsScene
import OutdoorsScene
from InputLayer import InputLayer
from Levels import Level01

from Context import *

class GameOverScene( cocos.scene.Scene ):
    def __init__( self, levelclass, level, won ):
        super( GameOverScene, self ).__init__()
        
        self.levelclass = levelclass
        self.level = level
        self.won = won
        
        self.input_layer = InputLayer()
        self.add( self.input_layer )
        self.do( Repeat( CallFunc( self.update ) ) )
        
        self.updateContext = Context(0.01, self.input_layer)
        
        # Background
        self.bglayer = cocos.layer.Layer()
        sprite = cocos.sprite.Sprite( "frontendsplash01.png" )
        sprite.position = 1200/2, 720/2
        self.bglayer.add( sprite )
        self.add( self.bglayer, z = Z.BG )
        
        texts = []
        
        if won:
            
            texts.append( 'Level complete!' )
            texts.append( 'Press space to move on to the next challenge!' )
        else:
            texts.append( 'Level failed!' )
            texts.append( 'Press space to retry.' )
        
        self.labels = []
        ypos = 330
        for t in texts:
            label = cocos.text.Label( t, font_size=18, x=350, y=ypos, color=(255,255,255,255) )
            self.labels.append( label )
            ypos -= 30
            self.add( label, z = 1.0 )
        
        
    def update( self ):
        
        
        if self.input_layer.is_key_down( key.SPACE ):
            if self.won:
                if self.level.getNextLevel() is not None:
                    go_scene = OutdoorsScene.OutdoorsScene( self.level.getNextLevel() )
                    tr = ZoomTransition(go_scene, duration=1)
                    cocos.director.director.replace( tr )
                else:
                    go_scene = CreditsScene.CreditsScene()
                    tr = ZoomTransition(go_scene, duration=1)
                    cocos.director.director.replace( tr )
            else:
                go_scene = OutdoorsScene.OutdoorsScene( self.levelclass )
                tr = ZoomTransition(go_scene, duration=1)
                cocos.director.director.replace( tr )
             
        self.input_layer.update()
        
        
        
        