import math
import random
import cocos

from cocos.scenes.transitions import *
from cocos.actions import *

from pyglet.window import key, mouse

from InputLayer import InputLayer
import Context
from Actors.Egg import Egg
from Actors.Bird import Bird

from Context import ActorBuildData, Animation, Z

from vec2d import Vec2d

import random


class MatingLayer( cocos.layer.base_layers.Layer ):
    
    is_event_handler = True
    
    def __init__( self, context ):
        super( MatingLayer, self ).__init__()
        self.selectedActors = []
        self.actor1 = self.actor2 = self.hoverActor = None
        self.context = context
        
        self.selectionDistance2 = 32**2
        
        genes = ["jumping", "flying", "nomminess", "reproducibility"]
        
        self.root = context.rootnode
        
        self.hudimg = cocos.sprite.Sprite( "geneHUD.png" )
        self.hudimg.x = 250
        self.hudimg.y = 700
        self.root.add(self.hudimg, z = Z.HUD + 0.001)
        
        self.starred_genes = []
        self.starred_icons = {}
        self.actorstars = []
        self.genestars = {}
        
        for istar in range(3):
            actorstar = cocos.sprite.Sprite( "star.png" )
            actorstar.visible=False
            self.actorstars.append(actorstar)
            self.root.add(actorstar, z = Z.BG + 0.01)
        
        x,y = 150, 700
        self.gene_icons = {}
        for g in genes:
            name = "icon_" + g
            s = cocos.sprite.Sprite( name + ".png" )
            s.x,s.y = x, y
            x += 40
            self.gene_icons[g] = s
            self.root.add(s, name=name,z=Z.HUD+0.1 )
            c=(0,0,0,255)
            l1 = cocos.text.Label( color = c, position = (0,-40), bold=True, font_name="Arial", font_size=14, anchor_x="center")
            l2 = cocos.text.Label( color = c, position = (0,-65), bold=True, font_name="Arial", font_size=14, anchor_x="center")
            l3 = cocos.text.Label( color = c, position = (0,-90), bold=True, font_name="Arial", font_size=14, anchor_x="center")
                        
            s.add(l1, name = "label1")
            s.add(l2, name = "label2")
            s.add(l3, name = "label3")
            
            star = cocos.sprite.Sprite( "star.png" )
            star.visible=False
            star.x, star.y = s.x,s.y
            self.starred_icons[g] = star
            
            star = cocos.sprite.Sprite( "star.png" )
            star.visible=False
            star.x = s.x + 2
            star.scale = 0.8
            self.genestars[g] = star
            self.root.add(star, z = Z.HUD + 0.09)
            
            
        #c=(0,0,0,255)
        #self.debuglabel = cocos.text.Label( color = c, position = (400), bold=True, font_name="Arial", font_size=14, anchor_x="center")
        
        self.actor1icon = cocos.sprite.Sprite( "selFirst.png" )
        self.actor2icon = cocos.sprite.Sprite( "selSecond.png" )
        self.actorHovericon = cocos.sprite.Sprite( "selHover.png" )
        self.actor1icon.x = 100
        self.actor2icon.x = 100
        self.actor1icon.y = 670
        self.actor2icon.y = 640
        self.actorHovericon.x = 100
        self.actor1icon.visible = False
        self.actor2icon.visible = False
        self.actorHovericon.visible = False
        self.root.add(self.actor1icon, z=Z.HUD + 0.1)
        self.root.add(self.actor2icon, z=Z.HUD + 0.1)
        self.root.add(self.actorHovericon, z=Z.HUD + 0.1)
            
        self.matet = 0.0
        self.mateButton = cocos.sprite.Sprite( "maeksecks.png" )
        self.mateButton.x,self.mateButton.y = 370,670
        self.mateButton.visible = False
        self.root.add(self.mateButton, z=Z.HUD + 0.1)
        
        
        self.hoverMarker = cocos.sprite.Sprite( "marker1.png" )
        self.hoverMarker.visible = False     
        self.hoverMarker.opacity = 100
        #self.root.add(self.hoverMarker, z = Z.BG + 0.01)
        
        self.mousepos = (0,0)
        
    def on_mouse_motion( self, x, y, dx, dy ):
        self.mousepos_rel = dx, dy
        self.mousepos = cocos.director.director.get_virtual_coordinates (x, y)
        d,a = self.context.level.findClosestActorByPos( Vec2d(self.mousepos), Bird )
        
        newhover = None
        if d < self.selectionDistance2 and a != self.actor1 and a != self.actor2:
            newhover = a

        if newhover != self.hoverActor:
            if self.hoverActor is not None:
                self.hoverActor.birdnode.remove("hovermarker")
            self.hoverActor = newhover
            if self.hoverActor is not None:
                self.hoverActor.birdnode.add(self.hoverMarker, name="hovermarker")

        
        self.updateGeneIcons()
        
        
    def on_mouse_press( self, x, y, buttons, modifier ):
        x1 = self.mateButton.x + 20
        y1 = self.mateButton.y 
        p = x1,y1
        if Vec2d(x,y).get_distance(Vec2d(p)) < 40 and self.mateButton.visible:
            if self.actor1 != None and self.actor2 != None: 
                self.mate()
            self.mateButton.visible = False
            return
    
        p = cocos.director.director.get_virtual_coordinates (x, y)
        p = Vec2d(p)
        
        for gene,icon in self.gene_icons.iteritems():
            if p.get_dist_sqrd(Vec2d(icon.x, icon.y)) < 20*20:
                starnode = self.starred_icons[gene]
                if starnode.visible:
                    self.root.remove(starnode)
                    starnode.visible = False
                    self.starred_genes.remove(gene) 
                else:
                    starnode.visible = True
                    self.starred_genes.append(gene)
                    self.root.add(starnode, z=Z.HUD+0.05)
        
        
        
        d,a = self.context.level.findClosestActorByPos( p, Bird )
        if d < self.selectionDistance2:
            self.toggleSelection( a )   
        else:
            self.deselect()
        if self.actor1 != None and self.actor2 != None:
            #self.mate()
            pass
    
    def deselect( self ):
        if self.actor1 != None:
            self.actor1.birdnode.remove( "marker" )
        if self.actor2 != None:
            self.actor2.birdnode.remove( "marker" )
            self.mateButton.visible = False
        self.actor1 = self.actor2 = None
        self.updateGeneIcons()
    
    def mate_gene(self, g1, g2):
        mean = (g1 + g2) * .5
        dist = 2 + abs(g1-g2) * .5
        g = random.gauss(mean, dist)
        return min(10.0, max(1.0, g ) )
    
    def mate( self ):
        
        self.context.audio.playSound("love")
        
        self.context.level.money -= 5
        
        self.actor1.disableMateing()
        self.actor2.disableMateing()
        build_data = ActorBuildData()
        
        build_data.pos = (Vec2d(self.actor1.x,self.actor1.y) + Vec2d(self.actor2.x, self.actor2.y)) * .5
        #build_data.image = "egg.png"
        
        build_data.genes = {}
        
        for gene in self.actor1.genes.keys():
            build_data.genes[gene] = self.mate_gene(self.actor1.genes[gene], self.actor2.genes[gene])
            
        self.context.level.createActor( Egg, self.context, build_data)
        self.deselect()
        
    def updateGeneIcons( self ):
        for k in self.gene_icons.keys():
            hoverplace = 1
            l = self.gene_icons[k].get("label1")
            if self.actor1 != None:
                l.element.text = "%i" % self.actor1.genes[k]
                hoverplace += 1
            else:
                l.element.text = ""
                
            l = self.gene_icons[k].get("label2")
            if self.actor2 != None:
                l.element.text = "%i" % self.actor2.genes[k]
                hoverplace += 1
            else:
                l.element.text = ""
                
            l = self.gene_icons[k].get("label"+str(hoverplace))
            if self.hoverActor != None:
                l.element.text = "%i" % self.hoverActor.genes[k]
            else:
                l.element.text = ""
        
        self.updateGeneStars()
        self.updateSelectionIcons()
            
    def mycmp(first, second):
        return cmp( first[0], second[0] )

    def updateGeneStars( self ):
        for k in self.gene_icons.keys():
            hoverplace = 1
            best = 0
            
            hoverplace = 1 + (self.actor1 is not None) + (self.actor2 is not None)
            actors = [(self.actor1, 1), (self.actor2, 2), (self.hoverActor, hoverplace)]
            scores = [(a[0].genes[k], a[1])  for a in actors if a[0] is not None]
            scores.sort()
            scores.reverse()
            
            star = self.genestars[k]
            numactors = len(scores)
            if numactors < 2 or scores[0][0] == scores[1][0]:
                star.visible = False
                continue
            
            starplace = scores[0][1]
            star.visible = True
            star.y = self.gene_icons[k].y - 7 - starplace * 25


    def updateSelectionIcons( self ):
        hoverplace = (self.actor1 is not None) + (self.actor2 is not None)
        
        self.actor1icon.visible = False
        self.actor2icon.visible = False
        self.actorHovericon.visible = False
        
        if self.actor1 != None:
            self.actor1icon.visible = True
            
        if self.actor2 != None:
            self.actor2icon.visible = True
            
        if self.hoverActor != None:
            self.actorHovericon.visible = True
            self.actorHovericon.y = 670 - hoverplace * 30
            

                

                    

            
    def toggleSelection( self, a ):
        
        if a == self.actor1:
            a.birdnode.remove( "marker" )
            self.actor1 = None
        elif a == self.actor2:
            a.birdnode.remove( "marker" ) 
            self.actor2 = None
        elif a.readyToMate:        
            s = cocos.sprite.Sprite( "marker1.png" )    
            a.birdnode.add(s, Z.BG + 0.01, name = "marker")
            if self.actor1 == None:
                self.actor1 = a
            elif self.actor2 == None:
                self.actor2 = a
            else:
                self.actor2.birdnode.remove( "marker" )
                self.actor2 = a
        self.mateButton.visible = self.actor1 != None and self.actor2 != None
        self.updateGeneIcons()
        
        
        
    def update(self, context):
        iconcount = (self.actor1 is not None) \
                    + (self.actor2 is not None) \
                    + (self.hoverActor is not None)
        targety =  760 - 25 * iconcount
        self.hudimg.y = self.hudimg.y + (targety - self.hudimg.y ) * 0.1
        
        best = []
        if self.starred_genes != []:
            for a in context.level.actors:
                if isinstance(a, Bird):
                    a_score = sum([a.genes[gene] for gene in self.starred_genes])
                    i = 0
                    for b in best:
                        b_score = sum([b.genes[gene] for gene in self.starred_genes])
                        if b_score < a_score:
                            break
                        i += 1
                        
                    best.insert(i,a)
                
        for star in self.actorstars:
            star.visible = False    
        
        i = 0
        numstars = min( len(self.actorstars), len([b for b in context.level.actors if isinstance(b,Bird)]) / 2)
        for b in best[:numstars ]:
            self.actorstars[i].x = b.birdnode.x
            self.actorstars[i].y = b.getRealY()
            self.actorstars[i].visible = True
            i += 1    
            
        
        if self.mateButton.visible:
            self.matet += context.dt
            #self.mateButton.rotation = math.sin(self.matet * 1) * 10
            self.mateButton.scale = 0.9 + 0.3 * math.sin(self.matet * 10)
        
        if self.hoverActor is None:
            self.hoverMarker.visible = False
        else:
            self.hoverMarker.visible = True
            #self.hoverMarker.x = self.hoverActor.x
            #self.hoverMarker.y = self.hoverActor.y
            
            